#!/bin/sh
#
# Configures Apollo for compilation
#
# The Apollo Team <team@apolloplayer.org>

PROJECT="Apollo"
CACHE=".cache"
LOGFILE=".log"
TMAKE_DEF="tmake"
QMAKE_DEF="qmake"
TMAKE=$TMAKE_DEF
PRO_FILE="apollo.pro"

BUILD="release"

# Read previous settings if any
if [ -f $CACHE ]; then
    echo "Reading cached settings"
    echo 
    . $CACHE
fi

# Check parameters
for arg in $*; do
    case $arg in
	--help|-h)
	    echo "Usage: $0 [options]"
	    echo
	    echo "Options: -h"
	    echo "         --help              This message"
	    echo "         --release           Build for release(default)"
	    echo "         --debug             Build for debug"
	    echo "         --with-tmake        Use TMake for making the Makefile (default)"
	    echo "         --with-qmake        Use QMake for making the Makefile (Qt3.0++)"
	    echo "         --with-qt-dir=DIR   Where the root of Qt is installed (default "'$QTDIR'")"
	    echo "         --with-stl[=DIR]    Add STL support with optional dir for include"
	    echo "         --detect-stl        Detect if STL can be used"
	    echo "         --without-stl       Do not add STL support"
	    echo "         --with-kde[=DIR]    Add KDE support with optional dir for include"
	    echo "         --without-kde       Do not add KDE support"
	    echo "         --detect-kde        Detect whether KDE can be used (default)"
	    echo "         --enable-buffer     Enable buffer setting for mpg123"
	    echo "         --disable-buffer    Disable buffer setting for mpg123 (default)"
	    echo "         --with-id3=DIR      Set id3lib directory"
	    echo "         --with-mad=DIR      Enable mad"
	    exit 1
	    ;;
	--release)
	    BUILD="release"
	    ;;
	--debug)
	    BUILD="debug"
	    ;;
	--with-tmake)
            TMAKE_SET=1
	    ;;
	--with-qmake)
            TMAKE_SET=2
	    ;;
	--with-qt-dir*)
	    if echo $arg | grep -e "--with-qt-dir=" >/dev/null; then
		QT_DIR=`echo $arg | sed 's/--with-qt-dir=/\1/'`
	    else
		echo "$arg: requires a directory"
                unset QT_DIR
	    fi
	    ;;
	--with-stl*)
	    if echo $arg | grep -e "--with-stl=" >/dev/null; then
		STL=`echo $arg | sed 's/--with-stl=/\1/'`
	    else
		STL="true"
	    fi
	    unset NO_STL
	    ;;
	--detect-stl)
	    unset STL
	    ;;
	--without-stl)
	    NO_STL="true"
	    ;;
	--with-kde*)
	    if echo $arg | grep -e "--with-kde=" >/dev/null; then
		KDE=`echo $arg | sed 's/--with-kde=/\1/'`
	    else
		KDE="true"
	    fi
	    unset NO_KDE
	    ;;
	--detect-kde)
	    unset KDE
	    unset NO_KDE
	    ;;
	--without-kde)
	    NO_KDE="true"
	    ;;
	--enable-buffer)
	    USE_BUFFER="true"
	    ;;
	--disable-buffer)
	    unset USE_BUFFER
	    ;;
	--with-id3*)
	    if echo $arg | grep -e "--with-id3=" >/dev/null; then
		ID3DIR=`echo $arg | sed 's/--with-id3=//'`
		if test ! -d $ID3DIR ; then
		    echo "id3 directory $ID3DIR does not exist"
		    unset ID3DIR
		fi
	    else
		unset ID3DIR
	    fi
	    ;;
	--with-mad*)
	    if echo $arg | grep -e "--with-mad=" >/dev/null; then
		MADDIR=`echo $arg | sed 's/--with-mad=//'`
		if test ! -d $MADDIR ; then
		    echo "mad directory $MADDIR does not exist"
		    MADDIR=""
		fi
	    else
		echo "$arg: requires a directory"
	    fi
	    ;;
	*)
	    echo "$arg: unkown option"
            $0 -h
	    exit 1
	    ;;
    esac;
done

case $TMAKE_SET in
    1)
	TMAKE=$TMAKE_DEF
	;;
    2)
	TMAKE=$QMAKE_DEF
	;;
esac

# Store the settings
if [ -f $CACHE ]; then
    rm $CACHE
fi
echo "BUILD=$BUILD" >> $CACHE
if [ ! -z $QT_DIR ]; then
    echo "QT_DIR=$QT_DIR" >> $CACHE
fi
if [ ! -z $STL ]; then
    echo "STL=$STL" >> $CACHE
fi
if [ ! -z $NO_STL ]; then
    echo "NO_STL=$NO_STL" >> $CACHE
fi
if [ ! -z $KDE ]; then
    echo "KDE=$KDE" >> $CACHE
fi
if [ ! -z $NO_KDE ]; then
    echo "NO_KDE=$NO_KDE" >> $CACHE
fi
if [ ! -z $USE_BUFFER ]; then
    echo "USE_BUFFER=$USE_BUFFER" >> $CACHE
fi
if [ ! -z $TMAKE_SET ]; then
    echo "TMAKE_SET=$TMAKE_SET" >> $CACHE
fi

# Setup tmake settings
TMAKE_CONF="CONFIG += $BUILD warn_off"

DEFTMAKEPATH="/usr/share/tmake"
DEFLOCALTMAKEPATH="/usr/local/share/tmake"
DEFTMAKELIBPATH="/usr/lib/tmake"
DEFTMAKELOCALLIBPATH="/usr/local/lib/tmake"

TMAKE_FOUND=

which $TMAKE >/dev/null

if [ $? -eq 1 ]; then
    if [ $TMAKE = $TMAKE_DEF ]; then
	which $QMAKE_DEF >/dev/null
	if [ $? -eq 0 ]; then
	    TMAKE_FOUND=true
  	    TMAKE=$QMAKE_DEF
	    echo "TMake not found, switching to QMake"
	fi
    else
	which $TMAKE_DEF >/dev/null
	if [ $? -eq 0 ]; then
	    TMAKE_FOUND=true
  	    TMAKE=$TMAKE_DEF
	    echo "QMake not found, switching to TMake"
	fi
    fi
else
    TMAKE_FOUND=true
fi

# Check for tmake
if [ "$TMAKE" == "$TMAKE_DEF" ]; then
    if [ -z $TMAKEPATH ]; then
	if [ ! -d "$DEFTMAKEPATH" -a ! -d "$DEFLOCALTMAKEPATH" -a ! -d "$DEFTMAKELIBPATH" -a ! -d "$DEFTMAKELOCALLIBPATH" ] ; then
	    echo
	    echo
	    echo '   The enviroment variable $TMAKEPATH is not set and the default'
	    echo "   tmake dirs $DEFTMAKEPATH, $DEFLOCALTMAKEPATH, $DEFTMAKELOCALLIBPATH or "
	    echo "   $DEFTMAKELIBPATH cannot be found."
	    echo '   This means that tmake is not installed or not properly setup.'
	    echo '   TMake requires this to be set to create the Makefile.'
	    echo '   You can get tmake from'
	    echo '    http://www.trolltech.com/products/download/freebies/tmake.html'
	    echo
	    echo
	    exit 1
	elif [ ! -f "$DEFTMAKEPATH/tmake.conf" -a ! -f "$DEFLOCALTMAKEPATH/tmake.conf" -a ! -f "$DEFTMAKELIBPATH/tmake.conf" -a ! -f "$DEFTMAKELOCALLIBPATH/tmake.conf" ] ; then
	    echo
	    echo    
	    echo '   The enviroment variable $TMAKEPATH is not set, but one of the default'
	    echo "   tmake dirs $DEFTMAKEPATH, $DEFLOCALTMAKEPATH, $DEFTMAKELOCALLIBPATH or "
	    echo "   $DEFTMAKELIBPATH was found.  Did NOT however find a tmake.conf, the "
	    echo "   tmake configuration file.  It might be in a platform dependent subdirectory"
	    echo "   such as linux-g++.  Find the directory, set $TMAKEPATH and run configure.sh"
	    echo "   again.  (export TMAKEPATH=/usr/lib/tmake/linux-g++/)."
	    echo
	    echo
	    exit 1
	fi
    fi
fi

if [ -z $TMAKE_FOUND ]; then
    echo
    echo
    echo "   TMake cannot be found in binary path, please make sure it's properly installed"
    echo '   You can get tmake from'
    echo '   http://www.trolltech.com/products/download/freebies/tmake.html'
    echo
    echo
    exit 1
fi

# Check if a userdefined qtdir is set
if [ -z $QT_DIR ]; then

# Check for Qt enviroment variables
    if [ -z $QTDIR ]; then
	echo
	echo
	echo '   The enviroment variable $QTDIR is not set. '$PROJECT' (and Qt programs) requires'
	echo '   this to be set to compile. Read the Qt documentation for more information.'
	echo
	echo
	exit 1
    fi
else
    if [ ! -z $QTDIR ]; then
	if [ $QT_DIR != $QTDIR ]; then
	    echo
	    echo
	    echo '    The enviroment variable $QTDIR is set but does not point to the directory'
	    echo '    specified in --with-qt-dir. '$PROJECT' will probably not compile correctly.'
	    echo '    Either set the enviroment variable $QTDIR to '$QT_DIR' by doing'
	    echo '      export QTDIR='$QT_DIR' (for bash compatible shells)'
	    echo '    or'
	    echo '      setenv QTDIR '$QT_DIR' (for csh compatible shells)'
	    echo
	    echo
	    exit 1
	fi
    else
	QTDIR=$QT_DIR
    fi
fi

# Find a make command
if ( make /dev/null ) >/dev/null 2>/dev/null
then
    MAKE=make
else
    if ( gmake /dev/null ) >/dev/null 2>/dev/null
    then
	MAKE=gmake
    else
	echo "You don't seem to have 'make' or 'gmake' in your PATH."
	echo "Cannot proceed."
	exit 1
    fi
fi

echo "Configuring $PROJECT:"
echo "Building for "$BUILD

rm -rf $LOGFILE
rm -rf $LOGFILE-kde
rm -rf $LOGFILE-stl
rm -rf $LOGFILE-qt
rm -rf $LOGFILE-id3

CHECK="Checking whether"
CHECK_FOR="Checking for"

INCLUDES=""
DEFINES=""
SOURCES=""
HEADERS=""
LIBRARIES=""
LIBDIRS=""

CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=
LIBS=""

CONF_LINK="g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS conftest.c $LIBS &>$LOGFILE-stl"

# Check STL support
if [ -z $NO_STL ]; then
    echo -n $CHECK "STL can be used... "
    if [ ! -z $STL ]; then
	if [ $STL != "true" ]; then
	    echo "Using STL in include path $STL"
	    INCLUDES="$INCLUDES $STL"
	else
	    echo "Using STL in standard include path"
	fi
	DEFINES="$DEFINES APOLLO_USE_STL"
    else
	cat > conftest.c <<EOF
#line 307 "configure.sh"
#include <memory>

int main( int, char ** )
{
    auto_ptr<int> tmp;
    tmp.reset( new int( 1 ) );
    int *ptr = tmp.get();
    return 0;
}
EOF
	eval $CONF_LINK
	if [ $? -eq 0 ]; then
	    DEFINES="$DEFINES APOLLO_USE_STL"
	    echo "STL support added"
	else
	    echo "STL cannot be used, using custom classes."
	    echo "It seems that you have not got an up-to-date STL version installed."
	    echo "If you wish to use STL in Apollo we recommend you get STL from"
	    echo "http://www.stlport.org"
	fi
	rm -rf conftest*
    fi
else
    echo "STL support disabled"
fi

CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=
LIBS="-L$QTDIR/lib -lqt"
INCS="-I$QTDIR/include"

CONF_LINK="g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS $INCS conftest.c $LIBS &>$LOGFILE-qt"

# Check Qt support
echo -n $CHECK_FOR "Qt... "
cat > conftest.c <<EOF
#line 345 "configure.sh"
#include <qapplication.h>

#if QT_VERSION < 200
#error "Apollo requires Qt version 2.00 or higher"
#endif

int main( int argc, char **argv ) {
    QApplication app( argc, argv );
    return 0;
    }
EOF
eval $CONF_LINK
if [ $? -eq 0 ]; then
    echo -n "yes, in $QTDIR"
else
    echo "no, Apollo cannot compile without Qt"
    echo "Please download Qt from http://www.trolltech.com before running $0 again"
    echo "Consult the file INSTALL for more information"
    exit 1
fi
echo $CONF_LINK >> $LOGFILE-qt
rm -rf conftest*

# Check Qt 3.0 support
cat > conftest.c <<EOF
#line 371 "configure.sh"
#include <qapplication.h>

#if QT_VERSION < 300
#error "Qt 3.0 build for Apollo is disabled"
#endif

int main( int argc, char **argv ) {
    QApplication app( argc, argv );
    return 0;
    }
EOF
eval $CONF_LINK
if [ $? -eq 0 ]; then
    echo " (Building for Qt 3.x)"
    DEFINES="$DEFINES USING_QT3"
else
    echo " (Building for Qt 2.x)"
fi
echo $CONF_LINK >> $LOGFILE-qt
rm -rf conftest*

CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=
LIBS="-L$KDEDIR/lib -L$QTDIR/lib -lkdecore -lDCOP -lqt"
INCS="-I$KDEDIR/include -I$KDEDIR/include/kde -I$QTDIR/include"

CONF_LINK="g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS $INCS conftest.c $LIBS &>$LOGFILE-kde"

# Check KDE support
if [ -z $NO_KDE ]; then
    echo -n $CHECK "KDE can be used... "
    if [ ! -z $KDE ]; then
	if [ $KDE != "true" ]; then
	    echo "Using KDE installed at $KDE"
	    INCLUDES="$INCLUDES $KDE"
	    KDEDIR=$KDE
	else
	    echo "Using KDE in include path $KDEDIR"
	fi
	USE_KDE="true"
    else
	if [ -z $KDEDIR ]; then
	    KDEDIR="/usr/local/kde"
	fi
	cat > conftest.c <<EOF
#line 417 "configure.sh"
#include <stdio.h>
#include <kapp.h>

int main() {
    KApplication app;
    return 0;
    }
EOF
	eval $CONF_LINK
	if [ $? -eq 0 ]; then
	    USE_KDE="true"
	    echo "KDE support added"
	else
	    echo "KDE support omitted"
	fi
	echo $CONF_LINK >> $LOGFILE-kde
  	rm -rf conftest*
    fi
else
    echo "KDE support disabled"
fi


# Check for id3lib
LIBS="-lz -lid3"
INCS=""
if [ ! -z "$ID3DIR" ]; then
    LIBS="-L$ID3DIR/lib $LIBS"
    INCS="-I$ID3DIR/include $INCS"
fi

cat > conftest.c <<EOF
#line 450 "configure.sh"
#include <id3/tag.h>

int main(int argc, char *argv[])
{
	ID3_Tag *test = new ID3_Tag();
	if ( ID3LIB_MAJOR_VERSION >= 3 && ID3LIB_MINOR_VERSION >= 8 )
		return 0;
	else
		return 1;
}
EOF

CFLAGS="-pipe"
CPPFLAGS=
LDFLAGS=

CONF_LINK="g++ -o conftest $CFLAGS $CPPFLAGS $LDFLAGS $INCS conftest.c $LIBS &>$LOGFILE-id3lib"

echo -n $CHECK_FOR "id3lib... "
eval $CONF_LINK
if test $? -eq 0 ; then
    ENABLE_ID3="true"
    echo -n "yes"
    LDPATH="/usr/local/lib"
    if [ ! -z "$ID3DIR" ]; then
	LDPATH="$LDPATH:$ID3DIR/lib"
    fi
    LDPATH="LD_LIBRARY_PATH=$LDPATH"
    PROGRUN="$LDPATH ./conftest"
    eval $PROGRUN
    if test $? -eq 0 ; then
	echo ""
    else
	echo " but insufficient version"
	echo
	echo
	echo '   ID3lib was found but had an insufficient version'
	echo '   The required version is 3.8.0 or higher.'
	echo '   You can get id3lib from'
	echo '    http://id3lib.sourceforge.net/'
	echo
	echo
	exit 1
    fi
else
    unset ENABLE_ID3
    echo "no"
    echo
    echo
    if [ ! -z "$ID3DIR" ]; then
	echo '   ID3lib was not found using the default include and library path.'
    else
	echo "   ID3lib was not found in $ID3DIR."
    fi
    echo '   You can get id3lib from'
    echo '    http://id3lib.sourceforge.net/'
    echo
    echo
    exit 1
fi
rm -rf conftest*

# Checks done

if [ ! -z $USE_KDE ]; then
    DEFINES="$DEFINES APOLLO_USE_KDE"
    SOURCES="$SOURCES kde/usekde.cpp kde/qmsystray_kde.cpp kde/qmapplication_kde.cpp"
    HEADERS="$HEADERS kde/usekde.h kde/qmsystray_kde.h kde/qmapplication_kde.h"
    LIBRARIES="$LIBRARIES -lkdecore -lDCOP -lksystemtrayapplet"
    LIBDIRS="$LIBDIRS -L$KDEDIR/lib"
    INCLUDES="$INCLUDES . $KDEDIR/include $KDEDIR/include/kde"
else
    SOURCES="$SOURCES qt/qmapplication_qt.cpp"
    HEADERS="$SOURCES qt/qmapplication_qt.h"
fi

if [ ! -z $USE_BUFFER ]; then
    DEFINES="$DEFINES ENABLE_MPG123_BUFFER"
fi

if test ! -z "$MADDIR" ; then
    INCLUDES="$INCLUDES $MADDIR/include"
    DEFINES="$DEFINES ENABLE_MAD"
    LIBRARIES="$LIBRARIES -lmad"
    LIBDIRS="$LIBDIRS -L$MADDIR/lib"
fi 

if test ! -z "$ENABLE_ID3" ; then 
    DEFINES="$DEFINES ENABLE_ID3"
    LIBRARIES="$LIBRARIES -lid3"
    if test ! -z "$ID3DIR" ; then
	INCLUDES="$INCLUDES $ID3DIR/include"
	LIBDIRS="$LIBDIRS -L$ID3DIR/lib"
    fi
fi

if [ ! -z $QT_DIR ]; then
    INCLUDES="$QT_DIR/include $INCLUDES"
fi

XMAKE_INCDIR_QT="TMAKE_INCDIR_QT"
XMAKE_CXXFLAGS="TMAKE_CXXFLAGS"
if [ "$TMAKE" == "$QMAKE_DEF" ]; then
    XMAKE_INCDIR_QT="QMAKE_INCDIR_QT"
    XMAKE_CXXFLAGS="QMAKE_CXXFLAGS_WARN_ON"
fi

echo 

TMAKE_INCLUDES="unix:$XMAKE_INCDIR_QT += $INCLUDES"
TMAKE_DEFINES="DEFINES += $DEFINES"
TMAKE_LIBS="LIBS += $LIBRARIES $LIBDIRS"
TMAKE_SOURCES="SOURCES += $SOURCES"
TMAKE_HEADERS="HEADERS += $HEADERS"
TMAKE_CXXFLAGS="unix:$XMAKE_CXXFLAGS += -Wall -Wno-ctor-dtor-privacy"

# Create the Makefile
echo -n "Creating Makefile"
if $TMAKE -o Makefile $PRO_FILE "$TMAKE_CONF" "$TMAKE_CXXFLAGS" "$TMAKE_INCLUDES" "$TMAKE_DEFINES" "$TMAKE_LIBS" "$TMAKE_SOURCES" "$TMAKE_HEADERS" &>.log-tmake; then
    rm -f .log-tmake
    echo 
    echo 
    echo $PROJECT is now configured for building. Just run $MAKE.
    echo
else
    echo " failed"
    echo "$TMAKE failed with error:"
    cat .log-tmake
fi

